from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Type

import System
import System.Collections
import System.Configuration
import System.Configuration.Assemblies
import System.Data
import System.IO
import System.Reflection
import System.Runtime
import System.Runtime.CompilerServices
import System.Runtime.InteropServices
import System.Runtime.Remoting
import System.Runtime.Serialization
import System.Security
import System.Security.AccessControl
import System.Security.Cryptography
import System.Security.Cryptography.X509Certificates
import System.Security.Policy
import System.Threading
import System.Threading.Tasks
import System.Xml
import System.Xml.Schema
import System.Xml.Serialization
from System.ComponentModel import MarshalByValueComponent


class CultureInfo:

    @property
    def Calendar(self): ...

    @Calendar.setter
    def Calendar(self): ...

    def ClearCachedData(self) -> None: ...

    def Clone(self) -> object: ...

    @property
    def CompareInfo(self): ...

    @CompareInfo.setter
    def CompareInfo(self): ...

    def CreateSpecificCulture(self, _: str) -> CultureInfo: ...

    @property
    def CultureTypes(self): ...

    @CultureTypes.setter
    def CultureTypes(self): ...

    def CurrentCulture(self, *args, **kwargs) -> Any: ...

    def CurrentUICulture(self, *args, **kwargs) -> Any: ...

    @property
    def DateTimeFormat(self): ...

    @DateTimeFormat.setter
    def DateTimeFormat(self): ...

    def DefaultThreadCurrentCulture(self, *args, **kwargs) -> Any: ...

    def DefaultThreadCurrentUICulture(self, *args, **kwargs) -> Any: ...

    @property
    def DisplayName(self): ...

    @DisplayName.setter
    def DisplayName(self): ...

    @property
    def EnglishName(self): ...

    @EnglishName.setter
    def EnglishName(self): ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetConsoleFallbackUICulture(self) -> CultureInfo: ...

    def GetCultureInfo(self, _: str, __: str) -> CultureInfo: ...

    def GetCultureInfoByIetfLanguageTag(self,
            _: str,
            ) -> CultureInfo: ...

    def GetCultures(self,
            _: CultureTypes,
            ) -> List[CultureInfo]: ...

    def GetFormat(self, _: Type) -> object: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    @property
    def IetfLanguageTag(self): ...

    @IetfLanguageTag.setter
    def IetfLanguageTag(self): ...

    def InstalledUICulture(self, *args, **kwargs) -> Any: ...

    def InvariantCulture(self, *args, **kwargs) -> Any: ...

    @property
    def IsNeutralCulture(self): ...

    @IsNeutralCulture.setter
    def IsNeutralCulture(self): ...

    @property
    def IsReadOnly(self): ...

    @IsReadOnly.setter
    def IsReadOnly(self): ...

    @property
    def KeyboardLayoutId(self): ...

    @KeyboardLayoutId.setter
    def KeyboardLayoutId(self): ...

    @property
    def LCID(self): ...

    @LCID.setter
    def LCID(self): ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Name(self): ...

    @Name.setter
    def Name(self): ...

    @property
    def NativeName(self): ...

    @NativeName.setter
    def NativeName(self): ...

    @property
    def NumberFormat(self): ...

    @NumberFormat.setter
    def NumberFormat(self): ...

    @property
    def OptionalCalendars(self): ...

    @OptionalCalendars.setter
    def OptionalCalendars(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    @property
    def Parent(self): ...

    @Parent.setter
    def Parent(self): ...

    def ReadOnly(self,
            _: CultureInfo,
            ) -> CultureInfo: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    @property
    def TextInfo(self): ...

    @TextInfo.setter
    def TextInfo(self): ...

    @property
    def ThreeLetterISOLanguageName(self): ...

    @ThreeLetterISOLanguageName.setter
    def ThreeLetterISOLanguageName(self): ...

    @property
    def ThreeLetterWindowsLanguageName(self): ...

    @ThreeLetterWindowsLanguageName.setter
    def ThreeLetterWindowsLanguageName(self): ...

    def ToString(self) -> str: ...

    @property
    def TwoLetterISOLanguageName(self): ...

    @TwoLetterISOLanguageName.setter
    def TwoLetterISOLanguageName(self): ...

    @property
    def UseUserOverride(self): ...

    @UseUserOverride.setter
    def UseUserOverride(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Calendar(self) -> Calendar: ...

    def get_CompareInfo(self) -> CompareInfo: ...

    def get_CultureTypes(self) -> CultureTypes: ...

    def get_CurrentCulture(self) -> CultureInfo: ...

    def get_CurrentUICulture(self) -> CultureInfo: ...

    def get_DateTimeFormat(self) -> DateTimeFormatInfo: ...

    def get_DefaultThreadCurrentCulture(self) -> CultureInfo: ...

    def get_DefaultThreadCurrentUICulture(self) -> CultureInfo: ...

    def get_DisplayName(self) -> str: ...

    def get_EnglishName(self) -> str: ...

    def get_IetfLanguageTag(self) -> str: ...

    def get_InstalledUICulture(self) -> CultureInfo: ...

    def get_InvariantCulture(self) -> CultureInfo: ...

    def get_IsNeutralCulture(self) -> bool: ...

    def get_IsReadOnly(self) -> bool: ...

    def get_KeyboardLayoutId(self) -> int: ...

    def get_LCID(self) -> int: ...

    def get_Name(self) -> str: ...

    def get_NativeName(self) -> str: ...

    def get_NumberFormat(self) -> NumberFormatInfo: ...

    def get_OptionalCalendars(self) -> List[Calendar]: ...

    def get_Parent(self) -> CultureInfo: ...

    def get_TextInfo(self) -> TextInfo: ...

    def get_ThreeLetterISOLanguageName(self) -> str: ...

    def get_ThreeLetterWindowsLanguageName(self) -> str: ...

    def get_TwoLetterISOLanguageName(self) -> str: ...

    def get_UseUserOverride(self) -> bool: ...

    def set_CurrentCulture(self, _: CultureInfo) -> None: ...

    def set_CurrentUICulture(self, _: CultureInfo) -> None: ...

    def set_DateTimeFormat(self, _: DateTimeFormatInfo) -> None: ...

    def set_DefaultThreadCurrentCulture(self,
            _: CultureInfo,
            ) -> None: ...

    def set_DefaultThreadCurrentUICulture(self,
            _: CultureInfo,
            ) -> None: ...

    def set_NumberFormat(self, _: NumberFormatInfo) -> None: ...


class DateTimeFormatInfo:

    @property
    def AMDesignator(self): ...

    @AMDesignator.setter
    def AMDesignator(self): ...

    @property
    def AbbreviatedDayNames(self): ...

    @AbbreviatedDayNames.setter
    def AbbreviatedDayNames(self): ...

    @property
    def AbbreviatedMonthGenitiveNames(self): ...

    @AbbreviatedMonthGenitiveNames.setter
    def AbbreviatedMonthGenitiveNames(self): ...

    @property
    def AbbreviatedMonthNames(self): ...

    @AbbreviatedMonthNames.setter
    def AbbreviatedMonthNames(self): ...

    @property
    def Calendar(self): ...

    @Calendar.setter
    def Calendar(self): ...

    @property
    def CalendarWeekRule(self): ...

    @CalendarWeekRule.setter
    def CalendarWeekRule(self): ...

    def Clone(self) -> object: ...

    def CurrentInfo(self, *args, **kwargs) -> Any: ...

    @property
    def DateSeparator(self): ...

    @DateSeparator.setter
    def DateSeparator(self): ...

    @property
    def DayNames(self): ...

    @DayNames.setter
    def DayNames(self): ...

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    @property
    def FirstDayOfWeek(self): ...

    @FirstDayOfWeek.setter
    def FirstDayOfWeek(self): ...

    @property
    def FullDateTimePattern(self): ...

    @FullDateTimePattern.setter
    def FullDateTimePattern(self): ...

    def GetAbbreviatedDayName(self, _: System.DayOfWeek) -> str: ...

    def GetAbbreviatedEraName(self, _: int) -> str: ...

    def GetAbbreviatedMonthName(self, _: int) -> str: ...

    def GetAllDateTimePatterns(self, _: str) -> List[str]: ...

    def GetDayName(self, _: System.DayOfWeek) -> str: ...

    def GetEra(self, _: str) -> int: ...

    def GetEraName(self, _: int) -> str: ...

    def GetFormat(self, _: Type) -> object: ...

    def GetHashCode(self) -> int: ...

    def GetInstance(self, _: Any) -> DateTimeFormatInfo: ...

    def GetMonthName(self, _: int) -> str: ...

    def GetShortestDayName(self, _: System.DayOfWeek) -> str: ...

    def GetType(self) -> Type: ...

    def InvariantInfo(self, *args, **kwargs) -> Any: ...

    @property
    def IsReadOnly(self): ...

    @IsReadOnly.setter
    def IsReadOnly(self): ...

    @property
    def LongDatePattern(self): ...

    @LongDatePattern.setter
    def LongDatePattern(self): ...

    @property
    def LongTimePattern(self): ...

    @LongTimePattern.setter
    def LongTimePattern(self): ...

    def MemberwiseClone(self) -> object: ...

    @property
    def MonthDayPattern(self): ...

    @MonthDayPattern.setter
    def MonthDayPattern(self): ...

    @property
    def MonthGenitiveNames(self): ...

    @MonthGenitiveNames.setter
    def MonthGenitiveNames(self): ...

    @property
    def MonthNames(self): ...

    @MonthNames.setter
    def MonthNames(self): ...

    @property
    def NativeCalendarName(self): ...

    @NativeCalendarName.setter
    def NativeCalendarName(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    @property
    def PMDesignator(self): ...

    @PMDesignator.setter
    def PMDesignator(self): ...

    @property
    def RFC1123Pattern(self): ...

    @RFC1123Pattern.setter
    def RFC1123Pattern(self): ...

    def ReadOnly(self,
            _: DateTimeFormatInfo,
            ) -> DateTimeFormatInfo: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def SetAllDateTimePatterns(self, _: List[str], __: str) -> None: ...

    @property
    def ShortDatePattern(self): ...

    @ShortDatePattern.setter
    def ShortDatePattern(self): ...

    @property
    def ShortTimePattern(self): ...

    @ShortTimePattern.setter
    def ShortTimePattern(self): ...

    @property
    def ShortestDayNames(self): ...

    @ShortestDayNames.setter
    def ShortestDayNames(self): ...

    @property
    def SortableDateTimePattern(self): ...

    @SortableDateTimePattern.setter
    def SortableDateTimePattern(self): ...

    @property
    def TimeSeparator(self): ...

    @TimeSeparator.setter
    def TimeSeparator(self): ...

    def ToString(self) -> str: ...

    @property
    def UniversalSortableDateTimePattern(self): ...

    @UniversalSortableDateTimePattern.setter
    def UniversalSortableDateTimePattern(self): ...

    @property
    def YearMonthPattern(self): ...

    @YearMonthPattern.setter
    def YearMonthPattern(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_AMDesignator(self) -> str: ...

    def get_AbbreviatedDayNames(self) -> List[str]: ...

    def get_AbbreviatedMonthGenitiveNames(self) -> List[str]: ...

    def get_AbbreviatedMonthNames(self) -> List[str]: ...

    def get_Calendar(self) -> Calendar: ...

    def get_CalendarWeekRule(self) -> CalendarWeekRule: ...

    def get_CurrentInfo(self) -> DateTimeFormatInfo: ...

    def get_DateSeparator(self) -> str: ...

    def get_DayNames(self) -> List[str]: ...

    def get_FirstDayOfWeek(self) -> System.DayOfWeek: ...

    def get_FullDateTimePattern(self) -> str: ...

    def get_InvariantInfo(self) -> DateTimeFormatInfo: ...

    def get_IsReadOnly(self) -> bool: ...

    def get_LongDatePattern(self) -> str: ...

    def get_LongTimePattern(self) -> str: ...

    def get_MonthDayPattern(self) -> str: ...

    def get_MonthGenitiveNames(self) -> List[str]: ...

    def get_MonthNames(self) -> List[str]: ...

    def get_NativeCalendarName(self) -> str: ...

    def get_PMDesignator(self) -> str: ...

    def get_RFC1123Pattern(self) -> str: ...

    def get_ShortDatePattern(self) -> str: ...

    def get_ShortTimePattern(self) -> str: ...

    def get_ShortestDayNames(self) -> List[str]: ...

    def get_SortableDateTimePattern(self) -> str: ...

    def get_TimeSeparator(self) -> str: ...

    def get_UniversalSortableDateTimePattern(self) -> str: ...

    def get_YearMonthPattern(self) -> str: ...

    def set_AMDesignator(self, _: str) -> None: ...

    def set_AbbreviatedDayNames(self, _: List[str]) -> None: ...

    def set_AbbreviatedMonthGenitiveNames(self, _: List[str]) -> None: ...

    def set_AbbreviatedMonthNames(self, _: List[str]) -> None: ...

    def set_Calendar(self, _: Calendar) -> None: ...

    def set_CalendarWeekRule(self, _: CalendarWeekRule) -> None: ...

    def set_DateSeparator(self, _: str) -> None: ...

    def set_DayNames(self, _: List[str]) -> None: ...

    def set_FirstDayOfWeek(self, _: System.DayOfWeek) -> None: ...

    def set_FullDateTimePattern(self, _: str) -> None: ...

    def set_LongDatePattern(self, _: str) -> None: ...

    def set_LongTimePattern(self, _: str) -> None: ...

    def set_MonthDayPattern(self, _: str) -> None: ...

    def set_MonthGenitiveNames(self, _: List[str]) -> None: ...

    def set_MonthNames(self, _: List[str]) -> None: ...

    def set_PMDesignator(self, _: str) -> None: ...

    def set_ShortDatePattern(self, _: str) -> None: ...

    def set_ShortTimePattern(self, _: str) -> None: ...

    def set_ShortestDayNames(self, _: List[str]) -> None: ...

    def set_TimeSeparator(self, _: str) -> None: ...

    def set_YearMonthPattern(self, _: str) -> None: ...


class NumberFormatInfo:

    def Clone(self) -> object: ...

    @property
    def CurrencyDecimalDigits(self): ...

    @CurrencyDecimalDigits.setter
    def CurrencyDecimalDigits(self): ...

    @property
    def CurrencyDecimalSeparator(self): ...

    @CurrencyDecimalSeparator.setter
    def CurrencyDecimalSeparator(self): ...

    @property
    def CurrencyGroupSeparator(self): ...

    @CurrencyGroupSeparator.setter
    def CurrencyGroupSeparator(self): ...

    @property
    def CurrencyGroupSizes(self): ...

    @CurrencyGroupSizes.setter
    def CurrencyGroupSizes(self): ...

    @property
    def CurrencyNegativePattern(self): ...

    @CurrencyNegativePattern.setter
    def CurrencyNegativePattern(self): ...

    @property
    def CurrencyPositivePattern(self): ...

    @CurrencyPositivePattern.setter
    def CurrencyPositivePattern(self): ...

    @property
    def CurrencySymbol(self): ...

    @CurrencySymbol.setter
    def CurrencySymbol(self): ...

    def CurrentInfo(self, *args, **kwargs) -> Any: ...

    @property
    def DigitSubstitution(self): ...

    @DigitSubstitution.setter
    def DigitSubstitution(self): ...

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetFormat(self, _: Type) -> object: ...

    def GetHashCode(self) -> int: ...

    def GetInstance(self, _: Any) -> NumberFormatInfo: ...

    def GetType(self) -> Type: ...

    def InvariantInfo(self, *args, **kwargs) -> Any: ...

    @property
    def IsReadOnly(self): ...

    @IsReadOnly.setter
    def IsReadOnly(self): ...

    def MemberwiseClone(self) -> object: ...

    @property
    def NaNSymbol(self): ...

    @NaNSymbol.setter
    def NaNSymbol(self): ...

    @property
    def NativeDigits(self): ...

    @NativeDigits.setter
    def NativeDigits(self): ...

    @property
    def NegativeInfinitySymbol(self): ...

    @NegativeInfinitySymbol.setter
    def NegativeInfinitySymbol(self): ...

    @property
    def NegativeSign(self): ...

    @NegativeSign.setter
    def NegativeSign(self): ...

    @property
    def NumberDecimalDigits(self): ...

    @NumberDecimalDigits.setter
    def NumberDecimalDigits(self): ...

    @property
    def NumberDecimalSeparator(self): ...

    @NumberDecimalSeparator.setter
    def NumberDecimalSeparator(self): ...

    @property
    def NumberGroupSeparator(self): ...

    @NumberGroupSeparator.setter
    def NumberGroupSeparator(self): ...

    @property
    def NumberGroupSizes(self): ...

    @NumberGroupSizes.setter
    def NumberGroupSizes(self): ...

    @property
    def NumberNegativePattern(self): ...

    @NumberNegativePattern.setter
    def NumberNegativePattern(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    @property
    def PerMilleSymbol(self): ...

    @PerMilleSymbol.setter
    def PerMilleSymbol(self): ...

    @property
    def PercentDecimalDigits(self): ...

    @PercentDecimalDigits.setter
    def PercentDecimalDigits(self): ...

    @property
    def PercentDecimalSeparator(self): ...

    @PercentDecimalSeparator.setter
    def PercentDecimalSeparator(self): ...

    @property
    def PercentGroupSeparator(self): ...

    @PercentGroupSeparator.setter
    def PercentGroupSeparator(self): ...

    @property
    def PercentGroupSizes(self): ...

    @PercentGroupSizes.setter
    def PercentGroupSizes(self): ...

    @property
    def PercentNegativePattern(self): ...

    @PercentNegativePattern.setter
    def PercentNegativePattern(self): ...

    @property
    def PercentPositivePattern(self): ...

    @PercentPositivePattern.setter
    def PercentPositivePattern(self): ...

    @property
    def PercentSymbol(self): ...

    @PercentSymbol.setter
    def PercentSymbol(self): ...

    @property
    def PositiveInfinitySymbol(self): ...

    @PositiveInfinitySymbol.setter
    def PositiveInfinitySymbol(self): ...

    @property
    def PositiveSign(self): ...

    @PositiveSign.setter
    def PositiveSign(self): ...

    def ReadOnly(self,
            _: NumberFormatInfo,
            ) -> NumberFormatInfo: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_CurrencyDecimalDigits(self) -> int: ...

    def get_CurrencyDecimalSeparator(self) -> str: ...

    def get_CurrencyGroupSeparator(self) -> str: ...

    def get_CurrencyGroupSizes(self) -> List[int]: ...

    def get_CurrencyNegativePattern(self) -> int: ...

    def get_CurrencyPositivePattern(self) -> int: ...

    def get_CurrencySymbol(self) -> str: ...

    def get_CurrentInfo(self) -> NumberFormatInfo: ...

    def get_DigitSubstitution(self) -> DigitShapes: ...

    def get_InvariantInfo(self) -> NumberFormatInfo: ...

    def get_IsReadOnly(self) -> bool: ...

    def get_NaNSymbol(self) -> str: ...

    def get_NativeDigits(self) -> List[str]: ...

    def get_NegativeInfinitySymbol(self) -> str: ...

    def get_NegativeSign(self) -> str: ...

    def get_NumberDecimalDigits(self) -> int: ...

    def get_NumberDecimalSeparator(self) -> str: ...

    def get_NumberGroupSeparator(self) -> str: ...

    def get_NumberGroupSizes(self) -> List[int]: ...

    def get_NumberNegativePattern(self) -> int: ...

    def get_PerMilleSymbol(self) -> str: ...

    def get_PercentDecimalDigits(self) -> int: ...

    def get_PercentDecimalSeparator(self) -> str: ...

    def get_PercentGroupSeparator(self) -> str: ...

    def get_PercentGroupSizes(self) -> List[int]: ...

    def get_PercentNegativePattern(self) -> int: ...

    def get_PercentPositivePattern(self) -> int: ...

    def get_PercentSymbol(self) -> str: ...

    def get_PositiveInfinitySymbol(self) -> str: ...

    def get_PositiveSign(self) -> str: ...

    def set_CurrencyDecimalDigits(self, _: int) -> None: ...

    def set_CurrencyDecimalSeparator(self, _: str) -> None: ...

    def set_CurrencyGroupSeparator(self, _: str) -> None: ...

    def set_CurrencyGroupSizes(self, _: List[int]) -> None: ...

    def set_CurrencyNegativePattern(self, _: int) -> None: ...

    def set_CurrencyPositivePattern(self, _: int) -> None: ...

    def set_CurrencySymbol(self, _: str) -> None: ...

    def set_DigitSubstitution(self, _: DigitShapes) -> None: ...

    def set_NaNSymbol(self, _: str) -> None: ...

    def set_NativeDigits(self, _: List[str]) -> None: ...

    def set_NegativeInfinitySymbol(self, _: str) -> None: ...

    def set_NegativeSign(self, _: str) -> None: ...

    def set_NumberDecimalDigits(self, _: int) -> None: ...

    def set_NumberDecimalSeparator(self, _: str) -> None: ...

    def set_NumberGroupSeparator(self, _: str) -> None: ...

    def set_NumberGroupSizes(self, _: List[int]) -> None: ...

    def set_NumberNegativePattern(self, _: int) -> None: ...

    def set_PerMilleSymbol(self, _: str) -> None: ...

    def set_PercentDecimalDigits(self, _: int) -> None: ...

    def set_PercentDecimalSeparator(self, _: str) -> None: ...

    def set_PercentGroupSeparator(self, _: str) -> None: ...

    def set_PercentGroupSizes(self, _: List[int]) -> None: ...

    def set_PercentNegativePattern(self, _: int) -> None: ...

    def set_PercentPositivePattern(self, _: int) -> None: ...

    def set_PercentSymbol(self, _: str) -> None: ...

    def set_PositiveInfinitySymbol(self, _: str) -> None: ...

    def set_PositiveSign(self, _: str) -> None: ...


class DateTimeStyles:

    def AdjustToUniversal(self, *args, **kwargs) -> Any: ...

    def AllowInnerWhite(self, *args, **kwargs) -> Any: ...

    def AllowLeadingWhite(self, *args, **kwargs) -> Any: ...

    def AllowTrailingWhite(self, *args, **kwargs) -> Any: ...

    def AllowWhiteSpaces(self, *args, **kwargs) -> Any: ...

    def AssumeLocal(self, *args, **kwargs) -> Any: ...

    def AssumeUniversal(self, *args, **kwargs) -> Any: ...

    def CompareTo(self, _: object) -> int: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def NoCurrentDateDefault(self, *args, **kwargs) -> Any: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def RoundtripKind(self, *args, **kwargs) -> Any: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class DigitShapes:

    def CompareTo(self, _: object) -> int: ...

    def Context(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def NativeNational(self, *args, **kwargs) -> Any: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class TimeSpanStyles:

    def AssumeNegative(self, *args, **kwargs) -> Any: ...

    def CompareTo(self, _: object) -> int: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Type

import System
import System.Collections
import System.Configuration
import System.Configuration.Assemblies
import System.Data
import System.Globalization
import System.IO
import System.Reflection
import System.Runtime
import System.Runtime.CompilerServices
import System.Runtime.InteropServices
import System.Runtime.Remoting
import System.Runtime.Serialization
import System.Security
import System.Security.AccessControl
import System.Security.Cryptography
import System.Security.Cryptography.X509Certificates
import System.Security.Policy
import System.Threading
import System.Threading.Tasks
import System.Xml
import System.Xml.Schema
import System.Xml.Serialization
from System.ComponentModel import MarshalByValueComponent


class X509Certificate:

    def CreateFromCertFile(self,
            _: str,
            ) -> System.Security.Cryptography.X509Certificates.X509Certificate: ...

    def CreateFromSignedFile(self,
            _: str,
            ) -> System.Security.Cryptography.X509Certificates.X509Certificate: ...

    def Dispose(self, _: bool) -> None: ...

    def Equals(self, _: object) -> bool: ...

    def Export(self,
            _: System.Security.Cryptography.X509Certificates.X509ContentType,
            __: str,
            ) -> List[bytes]: ...

    def Finalize(self) -> None: ...

    def FormatDate(self, _: System.DateTime) -> str: ...

    def GetCertHash(self,
            _: System.Security.Cryptography.HashAlgorithmName,
            ) -> List[bytes]: ...

    def GetCertHashString(self,
            _: System.Security.Cryptography.HashAlgorithmName,
            ) -> str: ...

    def GetEffectiveDateString(self) -> str: ...

    def GetExpirationDateString(self) -> str: ...

    def GetFormat(self) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetIssuerName(self) -> str: ...

    def GetKeyAlgorithm(self) -> str: ...

    def GetKeyAlgorithmParameters(self) -> List[bytes]: ...

    def GetKeyAlgorithmParametersString(self) -> str: ...

    def GetName(self) -> str: ...

    def GetPublicKey(self) -> List[bytes]: ...

    def GetPublicKeyString(self) -> str: ...

    def GetRawCertData(self) -> List[bytes]: ...

    def GetRawCertDataString(self) -> str: ...

    def GetSerialNumber(self) -> List[bytes]: ...

    def GetSerialNumberString(self) -> str: ...

    def GetType(self) -> Type: ...

    @property
    def Handle(self): ...

    @Handle.setter
    def Handle(self): ...

    def Import(self,
            _: str,
            __: str,
            ___: System.Security.Cryptography.X509Certificates.X509KeyStorageFlags,
            ) -> None: ...

    @property
    def Issuer(self): ...

    @Issuer.setter
    def Issuer(self): ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Reset(self) -> None: ...

    @property
    def Subject(self): ...

    @Subject.setter
    def Subject(self): ...

    def ToString(self, _: bool) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Handle(self) -> Any: ...

    def get_Issuer(self) -> str: ...

    def get_Subject(self) -> str: ...


class X509ContentType:

    def Authenticode(self, *args, **kwargs) -> Any: ...

    def Cert(self, *args, **kwargs) -> Any: ...

    def CompareTo(self, _: object) -> int: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def Pfx(self, *args, **kwargs) -> Any: ...

    def Pkcs12(self, *args, **kwargs) -> Any: ...

    def Pkcs7(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def SerializedCert(self, *args, **kwargs) -> Any: ...

    def SerializedStore(self, *args, **kwargs) -> Any: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def Unknown(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class X509KeyStorageFlags:

    def CompareTo(self, _: object) -> int: ...

    def DefaultKeySet(self, *args, **kwargs) -> Any: ...

    def EphemeralKeySet(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Exportable(self, *args, **kwargs) -> Any: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def MachineKeySet(self, *args, **kwargs) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def PersistKeySet(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def UserKeySet(self, *args, **kwargs) -> Any: ...

    def UserProtected(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Type

import System
import System.Collections
import System.Configuration
import System.Configuration.Assemblies
import System.Data
import System.Globalization
import System.IO
import System.Reflection
import System.Runtime
import System.Runtime.CompilerServices
import System.Runtime.InteropServices
import System.Runtime.Remoting
import System.Runtime.Serialization
import System.Security
import System.Security.AccessControl
import System.Security.Cryptography
import System.Security.Cryptography.X509Certificates
import System.Security.Policy
import System.Threading
import System.Threading.Tasks
import System.Xml
import System.Xml.Schema
import System.Xml.Serialization
from System.ComponentModel import MarshalByValueComponent


class HashAlgorithmName:

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MD5(self, *args, **kwargs) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Name(self): ...

    @Name.setter
    def Name(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def SHA1(self, *args, **kwargs) -> Any: ...

    def SHA256(self, *args, **kwargs) -> Any: ...

    def SHA384(self, *args, **kwargs) -> Any: ...

    def SHA512(self, *args, **kwargs) -> Any: ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_MD5(self) -> System.Security.Cryptography.HashAlgorithmName: ...

    def get_Name(self) -> str: ...

    def get_SHA1(self) -> System.Security.Cryptography.HashAlgorithmName: ...

    def get_SHA256(self) -> System.Security.Cryptography.HashAlgorithmName: ...

    def get_SHA384(self) -> System.Security.Cryptography.HashAlgorithmName: ...

    def get_SHA512(self) -> System.Security.Cryptography.HashAlgorithmName: ...

    def op_Equality(self,
            _: System.Security.Cryptography.HashAlgorithmName,
            __: System.Security.Cryptography.HashAlgorithmName,
            ) -> bool: ...

    def op_Inequality(self,
            _: System.Security.Cryptography.HashAlgorithmName,
            __: System.Security.Cryptography.HashAlgorithmName,
            ) -> bool: ...



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Type

import System
import System.Collections
import System.Configuration
import System.Configuration.Assemblies
import System.Data
import System.Globalization
import System.IO
import System.Reflection
import System.Runtime
import System.Runtime.CompilerServices
import System.Runtime.InteropServices
import System.Runtime.Remoting
import System.Runtime.Serialization
import System.Security
import System.Security.AccessControl
import System.Security.Cryptography
import System.Security.Cryptography.X509Certificates
import System.Security.Policy
import System.Threading
import System.Threading.Tasks
import System.Xml
import System.Xml.Schema
import System.Xml.Serialization
from System.ComponentModel import MarshalByValueComponent


class TypeCode:

    def Boolean(self, *args, **kwargs) -> Any: ...

    def Byte(self, *args, **kwargs) -> Any: ...

    def Char(self, *args, **kwargs) -> Any: ...

    def CompareTo(self, _: object) -> int: ...

    def DBNull(self, *args, **kwargs) -> Any: ...

    def DateTime(self, *args, **kwargs) -> Any: ...

    def Decimal(self, *args, **kwargs) -> Any: ...

    def Double(self, *args, **kwargs) -> Any: ...

    def Empty(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def Int16(self, *args, **kwargs) -> Any: ...

    def Int32(self, *args, **kwargs) -> Any: ...

    def Int64(self, *args, **kwargs) -> Any: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def Object(self, *args, **kwargs) -> Any: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def SByte(self, *args, **kwargs) -> Any: ...

    def Single(self, *args, **kwargs) -> Any: ...

    def String(self, *args, **kwargs) -> Any: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def UInt16(self, *args, **kwargs) -> Any: ...

    def UInt32(self, *args, **kwargs) -> Any: ...

    def UInt64(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class Predicate:

    def BeginInvoke(self, _: Any, __: Any, ___: object) -> Any: ...

    def Clone(self) -> object: ...

    def Combine(self, _: Any, __: Any) -> Any: ...

    def CombineImpl(self, _: Any) -> Any: ...

    def CreateDelegate(self,
            _: Type,
            __: object,
            ___: str,
            ____: bool,
            _____: bool,
            ) -> Any: ...

    def DynamicInvoke(self, _: List[object]) -> object: ...

    def DynamicInvokeImpl(self, _: List[object]) -> object: ...

    def EndInvoke(self, _: Any) -> bool: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetInvocationList(self) -> List[Any]: ...

    def GetMethodImpl(self) -> System.Reflection.MethodInfo: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def Invoke(self, _: Any) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Method(self): ...

    @Method.setter
    def Method(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Remove(self, _: Any, __: Any) -> Any: ...

    def RemoveAll(self, _: Any, __: Any) -> Any: ...

    def RemoveImpl(self, _: Any) -> Any: ...

    @property
    def Target(self): ...

    @Target.setter
    def Target(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Method(self) -> System.Reflection.MethodInfo: ...

    def get_Target(self) -> object: ...

    def op_Equality(self, _: Any, __: Any) -> bool: ...

    def op_Inequality(self, _: Any, __: Any) -> bool: ...


class TimeSpan:

    def Add(self, _: System.TimeSpan) -> System.TimeSpan: ...

    def Compare(self, _: System.TimeSpan, __: System.TimeSpan) -> int: ...

    def CompareTo(self, _: object) -> int: ...

    @property
    def Days(self): ...

    @Days.setter
    def Days(self): ...

    def Duration(self) -> System.TimeSpan: ...

    def Equals(self, _: System.TimeSpan, __: System.TimeSpan) -> bool: ...

    def Finalize(self) -> None: ...

    def FromDays(self, _: float) -> System.TimeSpan: ...

    def FromHours(self, _: float) -> System.TimeSpan: ...

    def FromMilliseconds(self, _: float) -> System.TimeSpan: ...

    def FromMinutes(self, _: float) -> System.TimeSpan: ...

    def FromSeconds(self, _: float) -> System.TimeSpan: ...

    def FromTicks(self, _: int) -> System.TimeSpan: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    @property
    def Hours(self): ...

    @Hours.setter
    def Hours(self): ...

    def MaxValue(self, *args, **kwargs) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Milliseconds(self): ...

    @Milliseconds.setter
    def Milliseconds(self): ...

    def MinValue(self, *args, **kwargs) -> Any: ...

    @property
    def Minutes(self): ...

    @Minutes.setter
    def Minutes(self): ...

    def Negate(self) -> System.TimeSpan: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: str, __: Any) -> System.TimeSpan: ...

    def ParseExact(self,
            _: str,
            __: str,
            ___: Any,
            ____: System.Globalization.TimeSpanStyles,
            ) -> System.TimeSpan: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    @property
    def Seconds(self): ...

    @Seconds.setter
    def Seconds(self): ...

    def Subtract(self, _: System.TimeSpan) -> System.TimeSpan: ...

    @property
    def Ticks(self): ...

    @Ticks.setter
    def Ticks(self): ...

    def TicksPerDay(self, *args, **kwargs) -> Any: ...

    def TicksPerHour(self, *args, **kwargs) -> Any: ...

    def TicksPerMillisecond(self, *args, **kwargs) -> Any: ...

    def TicksPerMinute(self, *args, **kwargs) -> Any: ...

    def TicksPerSecond(self, *args, **kwargs) -> Any: ...

    def ToString(self, _: str, __: Any) -> str: ...

    @property
    def TotalDays(self): ...

    @TotalDays.setter
    def TotalDays(self): ...

    @property
    def TotalHours(self): ...

    @TotalHours.setter
    def TotalHours(self): ...

    @property
    def TotalMilliseconds(self): ...

    @TotalMilliseconds.setter
    def TotalMilliseconds(self): ...

    @property
    def TotalMinutes(self): ...

    @TotalMinutes.setter
    def TotalMinutes(self): ...

    @property
    def TotalSeconds(self): ...

    @TotalSeconds.setter
    def TotalSeconds(self): ...

    def TryParse(self, _: str, __: Any, ___: System.TimeSpan) -> bool: ...

    def TryParseExact(self,
            _: str,
            __: str,
            ___: Any,
            ____: System.Globalization.TimeSpanStyles,
            _____: System.TimeSpan,
            ) -> bool: ...

    def Zero(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Days(self) -> int: ...

    def get_Hours(self) -> int: ...

    def get_Milliseconds(self) -> int: ...

    def get_Minutes(self) -> int: ...

    def get_Seconds(self) -> int: ...

    def get_Ticks(self) -> int: ...

    def get_TotalDays(self) -> float: ...

    def get_TotalHours(self) -> float: ...

    def get_TotalMilliseconds(self) -> float: ...

    def get_TotalMinutes(self) -> float: ...

    def get_TotalSeconds(self) -> float: ...

    def op_Addition(self, _: System.TimeSpan, __: System.TimeSpan) -> System.TimeSpan: ...

    def op_Equality(self, _: System.TimeSpan, __: System.TimeSpan) -> bool: ...

    def op_GreaterThan(self, _: System.TimeSpan, __: System.TimeSpan) -> bool: ...

    def op_GreaterThanOrEqual(self, _: System.TimeSpan, __: System.TimeSpan) -> bool: ...

    def op_Inequality(self, _: System.TimeSpan, __: System.TimeSpan) -> bool: ...

    def op_LessThan(self, _: System.TimeSpan, __: System.TimeSpan) -> bool: ...

    def op_LessThanOrEqual(self, _: System.TimeSpan, __: System.TimeSpan) -> bool: ...

    def op_Subtraction(self,
            _: System.TimeSpan,
            __: System.TimeSpan,
            ) -> System.TimeSpan: ...

    def op_UnaryNegation(self, _: System.TimeSpan) -> System.TimeSpan: ...

    def op_UnaryPlus(self, _: System.TimeSpan) -> System.TimeSpan: ...


class ModuleHandle:

    def EmptyHandle(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetRuntimeFieldHandleFromMetadataToken(self,
            _: int,
            ) -> System.RuntimeFieldHandle: ...

    def GetRuntimeMethodHandleFromMetadataToken(self,
            _: int,
            ) -> System.RuntimeMethodHandle: ...

    def GetRuntimeTypeHandleFromMetadataToken(self,
            _: int,
            ) -> System.RuntimeTypeHandle: ...

    def GetType(self) -> Type: ...

    @property
    def MDStreamVersion(self): ...

    @MDStreamVersion.setter
    def MDStreamVersion(self): ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ResolveFieldHandle(self,
            _: int,
            __: List[System.RuntimeTypeHandle],
            ___: List[System.RuntimeTypeHandle],
            ) -> System.RuntimeFieldHandle: ...

    def ResolveMethodHandle(self,
            _: int,
            __: List[System.RuntimeTypeHandle],
            ___: List[System.RuntimeTypeHandle],
            ) -> System.RuntimeMethodHandle: ...

    def ResolveTypeHandle(self,
            _: int,
            __: List[System.RuntimeTypeHandle],
            ___: List[System.RuntimeTypeHandle],
            ) -> System.RuntimeTypeHandle: ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_MDStreamVersion(self) -> int: ...

    def op_Equality(self, _: System.ModuleHandle, __: System.ModuleHandle) -> bool: ...

    def op_Inequality(self, _: System.ModuleHandle, __: System.ModuleHandle) -> bool: ...


class Delegate:

    def Clone(self) -> object: ...

    def Combine(self, _: Any, __: Any) -> Any: ...

    def CombineImpl(self, _: Any) -> Any: ...

    def CreateDelegate(self,
            _: Type,
            __: object,
            ___: str,
            ____: bool,
            _____: bool,
            ) -> Any: ...

    def DynamicInvoke(self, _: List[object]) -> object: ...

    def DynamicInvokeImpl(self, _: List[object]) -> object: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetInvocationList(self) -> List[Any]: ...

    def GetMethodImpl(self) -> System.Reflection.MethodInfo: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Method(self): ...

    @Method.setter
    def Method(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Remove(self, _: Any, __: Any) -> Any: ...

    def RemoveAll(self, _: Any, __: Any) -> Any: ...

    def RemoveImpl(self, _: Any) -> Any: ...

    @property
    def Target(self): ...

    @Target.setter
    def Target(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Method(self) -> System.Reflection.MethodInfo: ...

    def get_Target(self) -> object: ...

    def op_Equality(self, _: Any, __: Any) -> bool: ...

    def op_Inequality(self, _: Any, __: Any) -> bool: ...


class IAsyncResult:

    @property
    def AsyncState(self): ...

    @AsyncState.setter
    def AsyncState(self): ...

    @property
    def AsyncWaitHandle(self): ...

    @AsyncWaitHandle.setter
    def AsyncWaitHandle(self): ...

    @property
    def CompletedSynchronously(self): ...

    @CompletedSynchronously.setter
    def CompletedSynchronously(self): ...

    @property
    def IsCompleted(self): ...

    @IsCompleted.setter
    def IsCompleted(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_AsyncState(self) -> object: ...

    def get_AsyncWaitHandle(self) -> System.Threading.WaitHandle: ...

    def get_CompletedSynchronously(self) -> bool: ...

    def get_IsCompleted(self) -> bool: ...


class MulticastDelegate:

    def Clone(self) -> object: ...

    def Combine(self, _: Any, __: Any) -> Any: ...

    def CombineImpl(self, _: Any) -> Any: ...

    def CreateDelegate(self,
            _: Type,
            __: object,
            ___: str,
            ____: bool,
            _____: bool,
            ) -> Any: ...

    def DynamicInvoke(self, _: List[object]) -> object: ...

    def DynamicInvokeImpl(self, _: List[object]) -> object: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetInvocationList(self) -> List[Any]: ...

    def GetMethodImpl(self) -> System.Reflection.MethodInfo: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Method(self): ...

    @Method.setter
    def Method(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Remove(self, _: Any, __: Any) -> Any: ...

    def RemoveAll(self, _: Any, __: Any) -> Any: ...

    def RemoveImpl(self, _: Any) -> Any: ...

    @property
    def Target(self): ...

    @Target.setter
    def Target(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Method(self) -> System.Reflection.MethodInfo: ...

    def get_Target(self) -> object: ...

    def op_Equality(self, _: Any, __: Any) -> bool: ...

    def op_Inequality(self, _: Any, __: Any) -> bool: ...


class EventHandler:

    def BeginInvoke(self,
            _: object,
            __: System.EventArgs,
            ___: Any,
            ____: object,
            ) -> Any: ...

    def Clone(self) -> object: ...

    def Combine(self, _: Any, __: Any) -> Any: ...

    def CombineImpl(self, _: Any) -> Any: ...

    def CreateDelegate(self,
            _: Type,
            __: object,
            ___: str,
            ____: bool,
            _____: bool,
            ) -> Any: ...

    def DynamicInvoke(self, _: List[object]) -> object: ...

    def DynamicInvokeImpl(self, _: List[object]) -> object: ...

    def EndInvoke(self, _: Any) -> None: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetInvocationList(self) -> List[Any]: ...

    def GetMethodImpl(self) -> System.Reflection.MethodInfo: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def Invoke(self, _: object, __: System.EventArgs) -> None: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Method(self): ...

    @Method.setter
    def Method(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Remove(self, _: Any, __: Any) -> Any: ...

    def RemoveAll(self, _: Any, __: Any) -> Any: ...

    def RemoveImpl(self, _: Any) -> Any: ...

    @property
    def Target(self): ...

    @Target.setter
    def Target(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Method(self) -> System.Reflection.MethodInfo: ...

    def get_Target(self) -> object: ...

    def op_Equality(self, _: Any, __: Any) -> bool: ...

    def op_Inequality(self, _: Any, __: Any) -> bool: ...


class IFormatProvider:

    def GetFormat(self, _: Type) -> object: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class AsyncCallback:

    def BeginInvoke(self, _: Any, __: Any, ___: object) -> Any: ...

    def Clone(self) -> object: ...

    def Combine(self, _: Any, __: Any) -> Any: ...

    def CombineImpl(self, _: Any) -> Any: ...

    def CreateDelegate(self,
            _: Type,
            __: object,
            ___: str,
            ____: bool,
            _____: bool,
            ) -> Any: ...

    def DynamicInvoke(self, _: List[object]) -> object: ...

    def DynamicInvokeImpl(self, _: List[object]) -> object: ...

    def EndInvoke(self, _: Any) -> None: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetInvocationList(self) -> List[Any]: ...

    def GetMethodImpl(self) -> System.Reflection.MethodInfo: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def Invoke(self, _: Any) -> None: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Method(self): ...

    @Method.setter
    def Method(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Remove(self, _: Any, __: Any) -> Any: ...

    def RemoveAll(self, _: Any, __: Any) -> Any: ...

    def RemoveImpl(self, _: Any) -> Any: ...

    @property
    def Target(self): ...

    @Target.setter
    def Target(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Method(self) -> System.Reflection.MethodInfo: ...

    def get_Target(self) -> object: ...

    def op_Equality(self, _: Any, __: Any) -> bool: ...

    def op_Inequality(self, _: Any, __: Any) -> bool: ...


class EventArgs:

    def Empty(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class DateTime:

    def Add(self, _: System.TimeSpan) -> System.DateTime: ...

    def AddDays(self, _: float) -> System.DateTime: ...

    def AddHours(self, _: float) -> System.DateTime: ...

    def AddMilliseconds(self, _: float) -> System.DateTime: ...

    def AddMinutes(self, _: float) -> System.DateTime: ...

    def AddMonths(self, _: int) -> System.DateTime: ...

    def AddSeconds(self, _: float) -> System.DateTime: ...

    def AddTicks(self, _: int) -> System.DateTime: ...

    def AddYears(self, _: int) -> System.DateTime: ...

    def Compare(self, _: System.DateTime, __: System.DateTime) -> int: ...

    def CompareTo(self, _: object) -> int: ...

    @property
    def Date(self): ...

    @Date.setter
    def Date(self): ...

    @property
    def Day(self): ...

    @Day.setter
    def Day(self): ...

    @property
    def DayOfWeek(self): ...

    @DayOfWeek.setter
    def DayOfWeek(self): ...

    @property
    def DayOfYear(self): ...

    @DayOfYear.setter
    def DayOfYear(self): ...

    def DaysInMonth(self, _: int, __: int) -> int: ...

    def Equals(self, _: System.DateTime, __: System.DateTime) -> bool: ...

    def Finalize(self) -> None: ...

    def FromBinary(self, _: int) -> System.DateTime: ...

    def FromFileTime(self, _: int) -> System.DateTime: ...

    def FromFileTimeUtc(self, _: int) -> System.DateTime: ...

    def FromOADate(self, _: float) -> System.DateTime: ...

    def GetDateTimeFormats(self, _: str, __: Any) -> List[str]: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    @property
    def Hour(self): ...

    @Hour.setter
    def Hour(self): ...

    def IsDaylightSavingTime(self) -> bool: ...

    def IsLeapYear(self, _: int) -> bool: ...

    @property
    def Kind(self): ...

    @Kind.setter
    def Kind(self): ...

    def MaxValue(self, *args, **kwargs) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Millisecond(self): ...

    @Millisecond.setter
    def Millisecond(self): ...

    def MinValue(self, *args, **kwargs) -> Any: ...

    @property
    def Minute(self): ...

    @Minute.setter
    def Minute(self): ...

    @property
    def Month(self): ...

    @Month.setter
    def Month(self): ...

    def Now(self, *args, **kwargs) -> Any: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self,
            _: str,
            __: Any,
            ___: System.Globalization.DateTimeStyles,
            ) -> System.DateTime: ...

    def ParseExact(self,
            _: str,
            __: str,
            ___: Any,
            ____: System.Globalization.DateTimeStyles,
            ) -> System.DateTime: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    @property
    def Second(self): ...

    @Second.setter
    def Second(self): ...

    def SpecifyKind(self,
            _: System.DateTime,
            __: System.DateTimeKind,
            ) -> System.DateTime: ...

    def Subtract(self, _: System.DateTime) -> System.TimeSpan: ...

    @property
    def Ticks(self): ...

    @Ticks.setter
    def Ticks(self): ...

    @property
    def TimeOfDay(self): ...

    @TimeOfDay.setter
    def TimeOfDay(self): ...

    def ToBinary(self) -> int: ...

    def ToFileTime(self) -> int: ...

    def ToFileTimeUtc(self) -> int: ...

    def ToLocalTime(self) -> System.DateTime: ...

    def ToLongDateString(self) -> str: ...

    def ToLongTimeString(self) -> str: ...

    def ToOADate(self) -> float: ...

    def ToShortDateString(self) -> str: ...

    def ToShortTimeString(self) -> str: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def ToUniversalTime(self) -> System.DateTime: ...

    def Today(self, *args, **kwargs) -> Any: ...

    def TryParse(self,
            _: str,
            __: Any,
            ___: System.Globalization.DateTimeStyles,
            ____: System.DateTime,
            ) -> bool: ...

    def TryParseExact(self,
            _: str,
            __: str,
            ___: Any,
            ____: System.Globalization.DateTimeStyles,
            _____: System.DateTime,
            ) -> bool: ...

    def UtcNow(self, *args, **kwargs) -> Any: ...

    @property
    def Year(self): ...

    @Year.setter
    def Year(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Date(self) -> System.DateTime: ...

    def get_Day(self) -> int: ...

    def get_DayOfWeek(self) -> System.DayOfWeek: ...

    def get_DayOfYear(self) -> int: ...

    def get_Hour(self) -> int: ...

    def get_Kind(self) -> System.DateTimeKind: ...

    def get_Millisecond(self) -> int: ...

    def get_Minute(self) -> int: ...

    def get_Month(self) -> int: ...

    def get_Now(self) -> System.DateTime: ...

    def get_Second(self) -> int: ...

    def get_Ticks(self) -> int: ...

    def get_TimeOfDay(self) -> System.TimeSpan: ...

    def get_Today(self) -> System.DateTime: ...

    def get_UtcNow(self) -> System.DateTime: ...

    def get_Year(self) -> int: ...

    def op_Addition(self, _: System.DateTime, __: System.TimeSpan) -> System.DateTime: ...

    def op_Equality(self, _: System.DateTime, __: System.DateTime) -> bool: ...

    def op_GreaterThan(self, _: System.DateTime, __: System.DateTime) -> bool: ...

    def op_GreaterThanOrEqual(self, _: System.DateTime, __: System.DateTime) -> bool: ...

    def op_Inequality(self, _: System.DateTime, __: System.DateTime) -> bool: ...

    def op_LessThan(self, _: System.DateTime, __: System.DateTime) -> bool: ...

    def op_LessThanOrEqual(self, _: System.DateTime, __: System.DateTime) -> bool: ...

    def op_Subtraction(self,
            _: System.DateTime,
            __: System.DateTime,
            ) -> System.TimeSpan: ...


class Guid:

    def CompareTo(self, _: object) -> int: ...

    def Empty(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def NewGuid(self) -> System.Guid: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: str) -> System.Guid: ...

    def ParseExact(self, _: str, __: str) -> System.Guid: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToByteArray(self) -> List[bytes]: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, _: str, __: System.Guid) -> bool: ...

    def TryParseExact(self, _: str, __: str, ___: System.Guid) -> bool: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def op_Equality(self, _: System.Guid, __: System.Guid) -> bool: ...

    def op_Inequality(self, _: System.Guid, __: System.Guid) -> bool: ...


class Converter:

    def BeginInvoke(self, _: Any, __: Any, ___: object) -> Any: ...

    def Clone(self) -> object: ...

    def Combine(self, _: Any, __: Any) -> Any: ...

    def CombineImpl(self, _: Any) -> Any: ...

    def CreateDelegate(self,
            _: Type,
            __: object,
            ___: str,
            ____: bool,
            _____: bool,
            ) -> Any: ...

    def DynamicInvoke(self, _: List[object]) -> object: ...

    def DynamicInvokeImpl(self, _: List[object]) -> object: ...

    def EndInvoke(self, _: Any) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetInvocationList(self) -> List[Any]: ...

    def GetMethodImpl(self) -> System.Reflection.MethodInfo: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def Invoke(self, _: Any) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Method(self): ...

    @Method.setter
    def Method(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Remove(self, _: Any, __: Any) -> Any: ...

    def RemoveAll(self, _: Any, __: Any) -> Any: ...

    def RemoveImpl(self, _: Any) -> Any: ...

    @property
    def Target(self): ...

    @Target.setter
    def Target(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Method(self) -> System.Reflection.MethodInfo: ...

    def get_Target(self) -> object: ...

    def op_Equality(self, _: Any, __: Any) -> bool: ...

    def op_Inequality(self, _: Any, __: Any) -> bool: ...


class RuntimeMethodHandle:

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetFunctionPointer(self) -> Any: ...

    def GetHashCode(self) -> int: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    @property
    def Value(self): ...

    @Value.setter
    def Value(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Value(self) -> Any: ...

    def op_Equality(self,
            _: System.RuntimeMethodHandle,
            __: System.RuntimeMethodHandle,
            ) -> bool: ...

    def op_Inequality(self,
            _: System.RuntimeMethodHandle,
            __: System.RuntimeMethodHandle,
            ) -> bool: ...


class RuntimeTypeHandle:

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetModuleHandle(self) -> System.ModuleHandle: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    @property
    def Value(self): ...

    @Value.setter
    def Value(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Value(self) -> Any: ...

    def op_Equality(self, _: System.RuntimeTypeHandle, __: object) -> bool: ...

    def op_Inequality(self, _: System.RuntimeTypeHandle, __: object) -> bool: ...


class DateTimeKind:

    def CompareTo(self, _: object) -> int: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def Local(self, *args, **kwargs) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def Unspecified(self, *args, **kwargs) -> Any: ...

    def Utc(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class RuntimeFieldHandle:

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    @property
    def Value(self): ...

    @Value.setter
    def Value(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Value(self) -> Any: ...

    def op_Equality(self,
            _: System.RuntimeFieldHandle,
            __: System.RuntimeFieldHandle,
            ) -> bool: ...

    def op_Inequality(self,
            _: System.RuntimeFieldHandle,
            __: System.RuntimeFieldHandle,
            ) -> bool: ...


class Version:

    @property
    def Build(self): ...

    @Build.setter
    def Build(self): ...

    def Clone(self) -> object: ...

    def CompareTo(self, _: object) -> int: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    @property
    def Major(self): ...

    @Major.setter
    def Major(self): ...

    @property
    def MajorRevision(self): ...

    @MajorRevision.setter
    def MajorRevision(self): ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Minor(self): ...

    @Minor.setter
    def Minor(self): ...

    @property
    def MinorRevision(self): ...

    @MinorRevision.setter
    def MinorRevision(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: str) -> System.Version: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    @property
    def Revision(self): ...

    @Revision.setter
    def Revision(self): ...

    def ToString(self, _: int) -> str: ...

    def TryParse(self, _: str, __: System.Version) -> bool: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Build(self) -> int: ...

    def get_Major(self) -> int: ...

    def get_MajorRevision(self) -> int: ...

    def get_Minor(self) -> int: ...

    def get_MinorRevision(self) -> int: ...

    def get_Revision(self) -> int: ...

    def op_Equality(self, _: System.Version, __: System.Version) -> bool: ...

    def op_GreaterThan(self, _: System.Version, __: System.Version) -> bool: ...

    def op_GreaterThanOrEqual(self, _: System.Version, __: System.Version) -> bool: ...

    def op_Inequality(self, _: System.Version, __: System.Version) -> bool: ...

    def op_LessThan(self, _: System.Version, __: System.Version) -> bool: ...

    def op_LessThanOrEqual(self, _: System.Version, __: System.Version) -> bool: ...


class ResolveEventArgs:

    def Empty(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Name(self): ...

    @Name.setter
    def Name(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    @property
    def RequestingAssembly(self): ...

    @RequestingAssembly.setter
    def RequestingAssembly(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Name(self) -> str: ...

    def get_RequestingAssembly(self) -> System.Reflection.Assembly: ...


class Action:

    def BeginInvoke(self, _: Any, __: object) -> Any: ...

    def Clone(self) -> object: ...

    def Combine(self, _: Any, __: Any) -> Any: ...

    def CombineImpl(self, _: Any) -> Any: ...

    def CreateDelegate(self,
            _: Type,
            __: object,
            ___: str,
            ____: bool,
            _____: bool,
            ) -> Any: ...

    def DynamicInvoke(self, _: List[object]) -> object: ...

    def DynamicInvokeImpl(self, _: List[object]) -> object: ...

    def EndInvoke(self, _: Any) -> None: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetInvocationList(self) -> List[Any]: ...

    def GetMethodImpl(self) -> System.Reflection.MethodInfo: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def Invoke(self) -> None: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Method(self): ...

    @Method.setter
    def Method(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Remove(self, _: Any, __: Any) -> Any: ...

    def RemoveAll(self, _: Any, __: Any) -> Any: ...

    def RemoveImpl(self, _: Any) -> Any: ...

    @property
    def Target(self): ...

    @Target.setter
    def Target(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Method(self) -> System.Reflection.MethodInfo: ...

    def get_Target(self) -> object: ...

    def op_Equality(self, _: Any, __: Any) -> bool: ...

    def op_Inequality(self, _: Any, __: Any) -> bool: ...


class Object:

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class Func:

    def BeginInvoke(self, _: Any, __: object) -> Any: ...

    def Clone(self) -> object: ...

    def Combine(self, _: Any, __: Any) -> Any: ...

    def CombineImpl(self, _: Any) -> Any: ...

    def CreateDelegate(self,
            _: Type,
            __: object,
            ___: str,
            ____: bool,
            _____: bool,
            ) -> Any: ...

    def DynamicInvoke(self, _: List[object]) -> object: ...

    def DynamicInvokeImpl(self, _: List[object]) -> object: ...

    def EndInvoke(self, _: Any) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetInvocationList(self) -> List[Any]: ...

    def GetMethodImpl(self) -> System.Reflection.MethodInfo: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetType(self) -> Type: ...

    def Invoke(self) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Method(self): ...

    @Method.setter
    def Method(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Remove(self, _: Any, __: Any) -> Any: ...

    def RemoveAll(self, _: Any, __: Any) -> Any: ...

    def RemoveImpl(self, _: Any) -> Any: ...

    @property
    def Target(self): ...

    @Target.setter
    def Target(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Method(self) -> System.Reflection.MethodInfo: ...

    def get_Target(self) -> object: ...

    def op_Equality(self, _: Any, __: Any) -> bool: ...

    def op_Inequality(self, _: Any, __: Any) -> bool: ...


class Attribute:

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetCustomAttribute(self,
            _: System.Reflection.ParameterInfo,
            __: Type,
            ___: bool,
            ) -> System.Attribute: ...

    def GetCustomAttributes(self,
            _: System.Reflection.ParameterInfo,
            __: Type,
            ___: bool,
            ) -> List[System.Attribute]: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def IsDefaultAttribute(self) -> bool: ...

    def IsDefined(self,
            _: System.Reflection.ParameterInfo,
            __: Type,
            ___: bool,
            ) -> bool: ...

    def Match(self, _: object) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    @property
    def TypeId(self): ...

    @TypeId.setter
    def TypeId(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_TypeId(self) -> object: ...


class DayOfWeek:

    def CompareTo(self, _: object) -> int: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def Friday(self, *args, **kwargs) -> Any: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def Monday(self, *args, **kwargs) -> Any: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Saturday(self, *args, **kwargs) -> Any: ...

    def Sunday(self, *args, **kwargs) -> Any: ...

    def Thursday(self, *args, **kwargs) -> Any: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def Tuesday(self, *args, **kwargs) -> Any: ...

    def Wednesday(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class DateTimeOffset:

    def Add(self, _: System.TimeSpan) -> System.DateTimeOffset: ...

    def AddDays(self, _: float) -> System.DateTimeOffset: ...

    def AddHours(self, _: float) -> System.DateTimeOffset: ...

    def AddMilliseconds(self, _: float) -> System.DateTimeOffset: ...

    def AddMinutes(self, _: float) -> System.DateTimeOffset: ...

    def AddMonths(self, _: int) -> System.DateTimeOffset: ...

    def AddSeconds(self, _: float) -> System.DateTimeOffset: ...

    def AddTicks(self, _: int) -> System.DateTimeOffset: ...

    def AddYears(self, _: int) -> System.DateTimeOffset: ...

    def Compare(self, _: System.DateTimeOffset, __: System.DateTimeOffset) -> int: ...

    def CompareTo(self, _: System.DateTimeOffset) -> int: ...

    @property
    def Date(self): ...

    @Date.setter
    def Date(self): ...

    @property
    def DateTime(self): ...

    @DateTime.setter
    def DateTime(self): ...

    @property
    def Day(self): ...

    @Day.setter
    def Day(self): ...

    @property
    def DayOfWeek(self): ...

    @DayOfWeek.setter
    def DayOfWeek(self): ...

    @property
    def DayOfYear(self): ...

    @DayOfYear.setter
    def DayOfYear(self): ...

    def Equals(self, _: System.DateTimeOffset, __: System.DateTimeOffset) -> bool: ...

    def EqualsExact(self, _: System.DateTimeOffset) -> bool: ...

    def Finalize(self) -> None: ...

    def FromFileTime(self, _: int) -> System.DateTimeOffset: ...

    def FromUnixTimeMilliseconds(self, _: int) -> System.DateTimeOffset: ...

    def FromUnixTimeSeconds(self, _: int) -> System.DateTimeOffset: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    @property
    def Hour(self): ...

    @Hour.setter
    def Hour(self): ...

    @property
    def LocalDateTime(self): ...

    @LocalDateTime.setter
    def LocalDateTime(self): ...

    def MaxValue(self, *args, **kwargs) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Millisecond(self): ...

    @Millisecond.setter
    def Millisecond(self): ...

    def MinValue(self, *args, **kwargs) -> Any: ...

    @property
    def Minute(self): ...

    @Minute.setter
    def Minute(self): ...

    @property
    def Month(self): ...

    @Month.setter
    def Month(self): ...

    def Now(self, *args, **kwargs) -> Any: ...

    @property
    def Offset(self): ...

    @Offset.setter
    def Offset(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self,
            _: str,
            __: Any,
            ___: System.Globalization.DateTimeStyles,
            ) -> System.DateTimeOffset: ...

    def ParseExact(self,
            _: str,
            __: str,
            ___: Any,
            ____: System.Globalization.DateTimeStyles,
            ) -> System.DateTimeOffset: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    @property
    def Second(self): ...

    @Second.setter
    def Second(self): ...

    def Subtract(self, _: System.DateTimeOffset) -> System.TimeSpan: ...

    @property
    def Ticks(self): ...

    @Ticks.setter
    def Ticks(self): ...

    @property
    def TimeOfDay(self): ...

    @TimeOfDay.setter
    def TimeOfDay(self): ...

    def ToFileTime(self) -> int: ...

    def ToLocalTime(self) -> System.DateTimeOffset: ...

    def ToOffset(self, _: System.TimeSpan) -> System.DateTimeOffset: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def ToUniversalTime(self) -> System.DateTimeOffset: ...

    def ToUnixTimeMilliseconds(self) -> int: ...

    def ToUnixTimeSeconds(self) -> int: ...

    def TryParse(self,
            _: str,
            __: Any,
            ___: System.Globalization.DateTimeStyles,
            ____: System.DateTimeOffset,
            ) -> bool: ...

    def TryParseExact(self,
            _: str,
            __: str,
            ___: Any,
            ____: System.Globalization.DateTimeStyles,
            _____: System.DateTimeOffset,
            ) -> bool: ...

    @property
    def UtcDateTime(self): ...

    @UtcDateTime.setter
    def UtcDateTime(self): ...

    def UtcNow(self, *args, **kwargs) -> Any: ...

    @property
    def UtcTicks(self): ...

    @UtcTicks.setter
    def UtcTicks(self): ...

    @property
    def Year(self): ...

    @Year.setter
    def Year(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Date(self) -> System.DateTime: ...

    def get_DateTime(self) -> System.DateTime: ...

    def get_Day(self) -> int: ...

    def get_DayOfWeek(self) -> System.DayOfWeek: ...

    def get_DayOfYear(self) -> int: ...

    def get_Hour(self) -> int: ...

    def get_LocalDateTime(self) -> System.DateTime: ...

    def get_Millisecond(self) -> int: ...

    def get_Minute(self) -> int: ...

    def get_Month(self) -> int: ...

    def get_Now(self) -> System.DateTimeOffset: ...

    def get_Offset(self) -> System.TimeSpan: ...

    def get_Second(self) -> int: ...

    def get_Ticks(self) -> int: ...

    def get_TimeOfDay(self) -> System.TimeSpan: ...

    def get_UtcDateTime(self) -> System.DateTime: ...

    def get_UtcNow(self) -> System.DateTimeOffset: ...

    def get_UtcTicks(self) -> int: ...

    def get_Year(self) -> int: ...

    def op_Addition(self,
            _: System.DateTimeOffset,
            __: System.TimeSpan,
            ) -> System.DateTimeOffset: ...

    def op_Equality(self, _: System.DateTimeOffset, __: System.DateTimeOffset) -> bool: ...

    def op_GreaterThan(self,
            _: System.DateTimeOffset,
            __: System.DateTimeOffset,
            ) -> bool: ...

    def op_GreaterThanOrEqual(self,
            _: System.DateTimeOffset,
            __: System.DateTimeOffset,
            ) -> bool: ...

    def op_Implicit(self, _: System.DateTime) -> System.DateTimeOffset: ...

    def op_Inequality(self,
            _: System.DateTimeOffset,
            __: System.DateTimeOffset,
            ) -> bool: ...

    def op_LessThan(self, _: System.DateTimeOffset, __: System.DateTimeOffset) -> bool: ...

    def op_LessThanOrEqual(self,
            _: System.DateTimeOffset,
            __: System.DateTimeOffset,
            ) -> bool: ...

    def op_Subtraction(self,
            _: System.DateTimeOffset,
            __: System.TimeSpan,
            ) -> System.DateTimeOffset: ...



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Type

import System
import System.Collections
import System.Configuration
import System.Configuration.Assemblies
import System.Data
import System.Globalization
import System.IO
import System.Reflection
import System.Runtime
import System.Runtime.CompilerServices
import System.Runtime.InteropServices
import System.Runtime.Serialization
import System.Security
import System.Security.AccessControl
import System.Security.Cryptography
import System.Security.Cryptography.X509Certificates
import System.Security.Policy
import System.Threading
import System.Threading.Tasks
import System.Xml
import System.Xml.Schema
import System.Xml.Serialization
from System.ComponentModel import MarshalByValueComponent


class ObjRef:

    @property
    def ChannelInfo(self): ...

    @ChannelInfo.setter
    def ChannelInfo(self): ...

    @property
    def EnvoyInfo(self): ...

    @EnvoyInfo.setter
    def EnvoyInfo(self): ...

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetObjectData(self,
            _: System.Runtime.Serialization.SerializationInfo,
            __: System.Runtime.Serialization.StreamingContext,
            ) -> None: ...

    def GetRealObject(self,
            _: System.Runtime.Serialization.StreamingContext,
            ) -> object: ...

    def GetType(self) -> Type: ...

    def IsFromThisAppDomain(self) -> bool: ...

    def IsFromThisProcess(self) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    @property
    def TypeInfo(self): ...

    @TypeInfo.setter
    def TypeInfo(self): ...

    @property
    def URI(self): ...

    @URI.setter
    def URI(self): ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_ChannelInfo(self) -> IChannelInfo: ...

    def get_EnvoyInfo(self) -> IEnvoyInfo: ...

    def get_TypeInfo(self) -> IRemotingTypeInfo: ...

    def get_URI(self) -> str: ...

    def set_ChannelInfo(self, _: IChannelInfo) -> None: ...

    def set_EnvoyInfo(self, _: IEnvoyInfo) -> None: ...

    def set_TypeInfo(self, _: IRemotingTypeInfo) -> None: ...

    def set_URI(self, _: str) -> None: ...



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Type

import System
import System.Collections
import System.Configuration
import System.Configuration.Assemblies
import System.Data
import System.Globalization
import System.IO
import System.Reflection
import System.Runtime
import System.Runtime.InteropServices
import System.Runtime.Remoting
import System.Runtime.Serialization
import System.Security
import System.Security.AccessControl
import System.Security.Cryptography
import System.Security.Cryptography.X509Certificates
import System.Security.Policy
import System.Threading
import System.Threading.Tasks
import System.Xml
import System.Xml.Schema
import System.Xml.Serialization
from System.ComponentModel import MarshalByValueComponent


class ConfiguredTaskAwaitable:

    def ConfiguredTaskAwaiter(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetAwaiter(self) -> Any: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class TaskAwaiter:

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetResult(self) -> None: ...

    def GetType(self) -> Type: ...

    @property
    def IsCompleted(self): ...

    @IsCompleted.setter
    def IsCompleted(self): ...

    def MemberwiseClone(self) -> object: ...

    def OnCompleted(self, _: System.Action) -> None: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    def UnsafeOnCompleted(self, _: System.Action) -> None: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_IsCompleted(self) -> bool: ...


class YieldAwaitable:

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetAwaiter(self) -> Any: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    def YieldAwaiter(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...



#!/usr/bin/env python3
#
#  makers.py
#
#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
#  DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
#  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
#  OR OTHER DEALINGS IN THE SOFTWARE.
#

# stdlib
import re
import string
from types import FunctionType, ModuleType
from typing import Any, Iterable, List, Optional

# 3rd party
import clr  # type: ignore
import isort
from domdf_python_tools.paths import PathPlus
from isort import Config

# this package
from dotnet_stub_builder.type_conversion import Converter
from dotnet_stub_builder.utils import dedup, is_dunder, tab_in

clr.AddReference("System")

# 3rd party
import System  # type: ignore

__all__ = ["make_imports", "make_module", "make_package", "walk_attrs"]

isort_config = Config(force_single_line=True)
method_alphabet = f"_{string.ascii_uppercase}{string.ascii_lowercase}0123456789"

SYSTEM_MODULES = [
		"System",
		"System.Collections",
		"System.Configuration",
		"System.Configuration.Assemblies",
		"System.Data",
		"System.Globalization",
		"System.IO",
		"System.Reflection",
		"System.Runtime",
		"System.Runtime.CompilerServices",
		"System.Runtime.InteropServices",
		"System.Runtime.Remoting",
		"System.Runtime.Serialization",
		"System.Security",
		"System.Security.AccessControl",
		"System.Security.Cryptography",
		"System.Security.Cryptography.X509Certificates",
		"System.Security.Policy",
		"System.Threading",
		"System.Threading.Tasks",
		"System.Xml",
		"System.Xml.Schema",
		"System.Xml.Serialization",
		]


def make_imports(current_module_name: str) -> List[str]:
	"""
	Create the imports for the top of a stub file.

	:param current_module_name: The name of the current module.

	:return:
	"""

	imports = [
			"from __future__ import annotations",
			'',
			"from enum import Enum",
			"from typing import Any, Dict, List, Type",
			'',
			]

	for m in SYSTEM_MODULES:
		if m != current_module_name:
			imports.append(f"import {m}")

	return imports


def make_module(
		name: str,
		module: ModuleType,
		attr_list: Iterable[str] = (),
		first_party_imports: Iterable[str] = (),
		converter=Converter()
		) -> bool:
	"""
	Create type stubs for a module.

	:param name: The name of the module.
	:param module: The module object.
	:param attr_list: A list of attributes to create stubs for.
	:param first_party_imports: A list of first-party imports to include at the top of the file.
	"""

	buf = []
	path = name.split(".")

	stubs_dir = PathPlus(f"{path[0]}-stubs")
	stubs_dir.maybe_make()
	(stubs_dir / "/".join(x for x in path[1:-1])).maybe_make(parents=True)
	stub_file = stubs_dir / "/".join(x for x in path[1:-1]) / f"{path[-1]}.pyi"

	for imp in (*make_imports(name), *first_party_imports):
		imp = re.sub(fr"import {name}\.([A-Za-z_]+)\.([A-Za-z_]+)\.([A-Za-z_]+)", r"from .\1.\2 import \3", imp)
		imp = re.sub(fr"import {name}\.([A-Za-z_]+)\.([A-Za-z_]+)", r"from .\1 import \2", imp)
		imp = re.sub(fr"import {name}\.([A-Za-z_]+)", r"from . import \1", imp)
		buf.append(imp)

	if name != "System.ComponentModel":
		if name == "System":
			buf.append("from .ComponentModel import MarshalByValueComponent")
		else:
			buf.append("from System.ComponentModel import MarshalByValueComponent")

	buf.append('')
	buf.append('')

	for attr_name in dedup(attr_list):
		stub_code = walk_attrs(module, attr_name, converter=converter)
		stub_code = stub_code.replace(f": {name}.", ': ')
		stub_code = stub_code.replace(f" -> {name}.", ' -> ')
		stub_code = stub_code.replace(f"[{name}.", '[')
		stub_code.replace("System.Collections.Generic.IDictionary[System.String,System.String]", "Any")

		buf.append(stub_code)

	stub_file.write_clean(isort.code("\n".join(buf), config=isort_config))

	return True


def make_package(
		name: str,
		module: ModuleType,
		attr_list: Iterable[str] = (),
		converter=Converter(),
		) -> bool:
	"""
	Create type stubs for a module.

	:param name: The name of the module.
	:param module: The module object.
	:param attr_list: A list of attributes to create stubs for.
	"""

	path = name.split(".")

	stubs_dir = PathPlus(f"{path[0]}-stubs")
	(stubs_dir / "/".join(x for x in path[1:])).maybe_make(parents=True)

	return make_module(f"{name}.__init__", module, attr_list, converter=converter)


def walk_attrs(module: ModuleType, attr_name, converter=Converter()) -> str:
	buf = []

	if not is_dunder(attr_name):
		obj = getattr(module, attr_name)

		# TODO: case where obj is not a class
		if not isinstance(obj, FunctionType):
			bases = []
			for base in obj.__bases__:
				if base not in {System.Object, object}:
					if base.__name__ in converter.type_mapping:
						bases.append(converter.type_mapping[base.__name__])
					else:
						bases.append(base.__name__)

			bases = list(filter(lambda x: x is Any, bases))

			if bases:
				buf.append(f"class {attr_name}({', '.join(bases)}):")
				buf.append('')
			else:
				buf.append(f"class {attr_name}:")
				buf.append('')

			child_attrs = dir(obj)
			if "__init__" not in child_attrs:
				child_attrs.append("__init__")

			child_attrs.sort(key=lambda attr: [method_alphabet.index(letter) for letter in attr])

			for child_attr_name in child_attrs:
				if (not is_dunder(child_attr_name) or child_attr_name == "__init__") and child_attr_name not in {
						"None",
						"value__",
						"AttrsImpl",
						"ClassImpl",
						"DefaultValueImpl",
						"MemberImpl",
						"NameImpl",
						"PositionImpl",
						}:

					try:
						child_obj = getattr(obj, child_attr_name)
					except TypeError as e:
						if str(e) in {
								"instance property must be accessed through a class instance",
								"property cannot be read",
								}:

							if buf[-1]:
								buf.append('')

							buf.extend([
									tab_in("@property"),
									tab_in(f"def {child_attr_name}(self): ..."),
									'',
									tab_in(f"@{child_attr_name}.setter"),
									tab_in(f"def {child_attr_name}(self, value): ..."),
									'',
									])
							continue

						elif str(e) == "instance attribute must be accessed through a class instance":
							print(f"{e.__class__.__name__}: '{e}' occurred for {attr_name}.{child_attr_name}")
							continue

						else:
							raise e

					# TODO: if isinstance(child_obj, FunctionType):

					doc: Optional[str] = child_obj.__doc__
					if doc in {int.__doc__, str.__doc__}:
						doc = None
					return_type = "Any"
					arguments: Optional[str] = None

					if doc:
						for line in doc.splitlines():
							m = re.match(fr"^(.*) {child_attr_name}\((.*)\)", line.strip())
							if m:
								csharp_return_type = m.group(1)
								return_type = converter.convert_type(csharp_return_type)
								arguments = m.group(2)

					if arguments is not None and arguments:
						signature = []

						for idx, argument in enumerate(arguments.split(", ")):
							signature.append(f"{'_' * (idx + 1)}: {converter.convert_type(argument)}")

						line = tab_in(f"def {child_attr_name}(self, {', '.join(signature)}) -> {return_type}: ...")
						if len(line) > 92:
							sig = ',\n        '.join(("self", *signature, ''))

							if buf[-1]:
								buf.append('')

							line = tab_in(f"def {child_attr_name}({sig}) -> {return_type}: ...")
							buf.append(line)
							buf.append('')
						else:
							buf.append(line)

					elif arguments is None:
						buf.append(tab_in(f"def {child_attr_name}(self, *args, **kwargs) -> {return_type}: ..."))

					elif not arguments:
						# i.e. takes no arguments
						buf.append(tab_in(f"def {child_attr_name}(self) -> {return_type}: ..."))

		buf.append("")

		return "\n".join(buf)

	return ''



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Type

import System
import System.Collections
import System.Configuration
import System.Configuration.Assemblies
import System.Data
import System.Globalization
import System.IO
import System.Reflection
import System.Runtime
import System.Runtime.CompilerServices
import System.Runtime.InteropServices
import System.Runtime.Remoting
import System.Runtime.Serialization
import System.Security
import System.Security.AccessControl
import System.Security.Cryptography
import System.Security.Cryptography.X509Certificates
import System.Security.Policy
import System.Threading
import System.Xml
import System.Xml.Schema
import System.Xml.Serialization
from System.ComponentModel import MarshalByValueComponent


class Task:

    @property
    def AsyncState(self): ...

    @AsyncState.setter
    def AsyncState(self): ...

    def CompletedTask(self, *args, **kwargs) -> Any: ...

    def ConfigureAwait(self,
            _: bool,
            ) -> System.Runtime.CompilerServices.ConfiguredTaskAwaitable: ...

    def ContinueWith(self,
            _: System.Action[Task, object],
            __: object,
            ___: System.Threading.CancellationToken,
            ____: TaskContinuationOptions,
            _____: TaskScheduler,
            ) -> Task: ...

    @property
    def CreationOptions(self): ...

    @CreationOptions.setter
    def CreationOptions(self): ...

    def CurrentId(self, *args, **kwargs) -> Any: ...

    def Delay(self,
            _: int,
            __: System.Threading.CancellationToken,
            ) -> Task: ...

    def Dispose(self, _: bool) -> None: ...

    def Equals(self, _: object, __: object) -> bool: ...

    @property
    def Exception(self): ...

    @Exception.setter
    def Exception(self): ...

    def Factory(self, *args, **kwargs) -> Any: ...

    def Finalize(self) -> None: ...

    def FromCanceled(self,
            _: System.Threading.CancellationToken,
            ) -> Task: ...

    def FromException(self, _: Exception) -> Task: ...

    def FromResult(self, *args, **kwargs) -> Any: ...

    def GetAwaiter(self) -> System.Runtime.CompilerServices.TaskAwaiter: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    @property
    def Id(self): ...

    @Id.setter
    def Id(self): ...

    @property
    def IsCanceled(self): ...

    @IsCanceled.setter
    def IsCanceled(self): ...

    @property
    def IsCompleted(self): ...

    @IsCompleted.setter
    def IsCompleted(self): ...

    @property
    def IsFaulted(self): ...

    @IsFaulted.setter
    def IsFaulted(self): ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Run(self,
            _: System.Func[Task],
            __: System.Threading.CancellationToken,
            ) -> Task: ...

    def RunSynchronously(self, _: TaskScheduler) -> None: ...

    def Start(self, _: TaskScheduler) -> None: ...

    @property
    def Status(self): ...

    @Status.setter
    def Status(self): ...

    def ToString(self) -> str: ...

    def Wait(self, _: int, __: System.Threading.CancellationToken) -> bool: ...

    def WaitAll(self,
            _: List[Task],
            __: int,
            ___: System.Threading.CancellationToken,
            ) -> bool: ...

    def WaitAny(self,
            _: List[Task],
            __: int,
            ___: System.Threading.CancellationToken,
            ) -> int: ...

    def WhenAll(self,
            _: List[Task],
            ) -> Task: ...

    def WhenAny(self,
            _: List[Task],
            ) -> Task[Task]: ...

    def Yield(self) -> System.Runtime.CompilerServices.YieldAwaitable: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_AsyncState(self) -> object: ...

    def get_CompletedTask(self) -> Task: ...

    def get_CreationOptions(self) -> TaskCreationOptions: ...

    def get_CurrentId(self) -> Any: ...

    def get_Exception(self) -> Exception: ...

    def get_Factory(self) -> TaskFactory: ...

    def get_Id(self) -> int: ...

    def get_IsCanceled(self) -> bool: ...

    def get_IsCompleted(self) -> bool: ...

    def get_IsFaulted(self) -> bool: ...

    def get_Status(self) -> TaskStatus: ...


class TaskScheduler:

    def Current(self, *args, **kwargs) -> Any: ...

    def Default(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    def FromCurrentSynchronizationContext(self) -> TaskScheduler: ...

    def GetHashCode(self) -> int: ...

    def GetScheduledTasks(self) -> List[Task]: ...

    def GetType(self) -> Type: ...

    @property
    def Id(self): ...

    @Id.setter
    def Id(self): ...

    @property
    def MaximumConcurrencyLevel(self): ...

    @MaximumConcurrencyLevel.setter
    def MaximumConcurrencyLevel(self): ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def QueueTask(self, _: Task) -> None: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def ToString(self) -> str: ...

    def TryDequeue(self, _: Task) -> bool: ...

    def TryExecuteTask(self, _: Task) -> bool: ...

    def TryExecuteTaskInline(self, _: Task, __: bool) -> bool: ...

    def UnobservedTaskException(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def add_UnobservedTaskException(self, _: Any) -> None: ...

    def get_Current(self) -> TaskScheduler: ...

    def get_Default(self) -> TaskScheduler: ...

    def get_Id(self) -> int: ...

    def get_MaximumConcurrencyLevel(self) -> int: ...

    def remove_UnobservedTaskException(self, _: Any) -> None: ...


class TaskFactory:

    @property
    def CancellationToken(self): ...

    @CancellationToken.setter
    def CancellationToken(self): ...

    @property
    def ContinuationOptions(self): ...

    @ContinuationOptions.setter
    def ContinuationOptions(self): ...

    def ContinueWhenAll(self,
            _: List[Task],
            __: Any,
            ___: System.Threading.CancellationToken,
            ____: TaskContinuationOptions,
            _____: TaskScheduler,
            ) -> Task: ...

    def ContinueWhenAny(self,
            _: List[Task],
            __: System.Action[Task],
            ___: System.Threading.CancellationToken,
            ____: TaskContinuationOptions,
            _____: TaskScheduler,
            ) -> Task: ...

    @property
    def CreationOptions(self): ...

    @CreationOptions.setter
    def CreationOptions(self): ...

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    def FromAsync(self,
            _: System.Func[Any, object, Any],
            __: System.Action[Any],
            ___: object,
            ____: TaskCreationOptions,
            ) -> Task: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    @property
    def Scheduler(self): ...

    @Scheduler.setter
    def Scheduler(self): ...

    def StartNew(self,
            _: System.Action[object],
            __: object,
            ___: System.Threading.CancellationToken,
            ____: TaskCreationOptions,
            _____: TaskScheduler,
            ) -> Task: ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_CancellationToken(self) -> System.Threading.CancellationToken: ...

    def get_ContinuationOptions(self) -> TaskContinuationOptions: ...

    def get_CreationOptions(self) -> TaskCreationOptions: ...

    def get_Scheduler(self) -> TaskScheduler: ...


class TaskCreationOptions:

    def AttachedToParent(self, *args, **kwargs) -> Any: ...

    def CompareTo(self, _: object) -> int: ...

    def DenyChildAttach(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def HideScheduler(self, *args, **kwargs) -> Any: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def LongRunning(self, *args, **kwargs) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def PreferFairness(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def RunContinuationsAsynchronously(self, *args, **kwargs) -> Any: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class TaskStatus:

    def Canceled(self, *args, **kwargs) -> Any: ...

    def CompareTo(self, _: object) -> int: ...

    def Created(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def Faulted(self, *args, **kwargs) -> Any: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def MemberwiseClone(self) -> object: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def RanToCompletion(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def Running(self, *args, **kwargs) -> Any: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def WaitingForActivation(self, *args, **kwargs) -> Any: ...

    def WaitingForChildrenToComplete(self, *args, **kwargs) -> Any: ...

    def WaitingToRun(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...


class UnobservedTaskExceptionEventArgs:

    def Empty(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object, __: object) -> bool: ...

    @property
    def Exception(self): ...

    @Exception.setter
    def Exception(self): ...

    def Finalize(self) -> None: ...

    def GetHashCode(self) -> int: ...

    def GetType(self) -> Type: ...

    def MemberwiseClone(self) -> object: ...

    @property
    def Observed(self): ...

    @Observed.setter
    def Observed(self): ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def SetObserved(self) -> None: ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Exception(self) -> Exception: ...

    def get_Observed(self) -> bool: ...


class TaskContinuationOptions:

    def AttachedToParent(self, *args, **kwargs) -> Any: ...

    def CompareTo(self, _: object) -> int: ...

    def DenyChildAttach(self, *args, **kwargs) -> Any: ...

    def Equals(self, _: object) -> bool: ...

    def ExecuteSynchronously(self, *args, **kwargs) -> Any: ...

    def Finalize(self) -> None: ...

    def Format(self, _: Type, __: object, ___: str) -> str: ...

    def GetHashCode(self) -> int: ...

    def GetName(self, _: Type, __: object) -> str: ...

    def GetNames(self, _: Type) -> List[str]: ...

    def GetType(self) -> Type: ...

    def GetTypeCode(self) -> Any: ...

    def GetUnderlyingType(self, _: Type) -> Type: ...

    def GetValues(self, _: Type) -> List: ...

    def HasFlag(self, _: Enum) -> bool: ...

    def HideScheduler(self, *args, **kwargs) -> Any: ...

    def IsDefined(self, _: Type, __: object) -> bool: ...

    def LazyCancellation(self, *args, **kwargs) -> Any: ...

    def LongRunning(self, *args, **kwargs) -> Any: ...

    def MemberwiseClone(self) -> object: ...

    def NotOnCanceled(self, *args, **kwargs) -> Any: ...

    def NotOnFaulted(self, *args, **kwargs) -> Any: ...

    def NotOnRanToCompletion(self, *args, **kwargs) -> Any: ...

    def OnlyOnCanceled(self, *args, **kwargs) -> Any: ...

    def OnlyOnFaulted(self, *args, **kwargs) -> Any: ...

    def OnlyOnRanToCompletion(self, *args, **kwargs) -> Any: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def Parse(self, _: Type, __: str, ___: bool) -> object: ...

    def PreferFairness(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def RunContinuationsAsynchronously(self, *args, **kwargs) -> Any: ...

    def ToObject(self, _: Type, __: object) -> object: ...

    def ToString(self, _: str, __: Any) -> str: ...

    def TryParse(self, *args, **kwargs) -> Any: ...

    def __init__(self, *args, **kwargs) -> Any: ...



#!/usr/bin/env python3
#
#  type_conversion.py
#
#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to deal
#  in the Software without restriction, including without limitation the rights
#  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#  copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in all
#  copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
#  DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
#  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
#  OR OTHER DEALINGS IN THE SOFTWARE.
#

# stdlib
import re
from typing import Dict, Optional

__all__ = ["generic_type_regex", "Converter"]

generic_type_regex = re.compile(r"^([A-Za-z_.]+)(`[0-9])?\[([A-Za-z_.,]+)]$")


class Converter:
	"""
	Class to convert .NET types into Python types.
	"""

	def __init__(self, extra_mappings: Optional[Dict[str, str]] = None):
		if extra_mappings is None:
			extra_mappings = {}

		# Mapping of .NET types to Python types (all as strings)
		self.type_mapping = {
				"Boolean": "bool",
				"System.Boolean": "bool",
				"Byte": "bytes",
				"SByte": "bytes",  # TODO
				"Char": "str",
				"System.Decimal": "float",  # TODO
				"Single": "float",
				"System.Single": "float",
				"Double": "float",
				"System.Double": "float",
				"Int32": "int",
				"UInt32": "int",  # TODO: sign
				"Int64": "int",  # TODO: bit size
				"UInt64": "int",  # TODO: bit size & sign
				# UInt64
				"Int16": "int",  # TODO: bit size
				"UInt16": "int",  # TODO: bit size & sign
				"System.Object": "object",
				"System.String": "str",
				"System.Collections.Generic.IDictionary": "Dict",
				"System.Type": "Type",
				"Long": "int",
				"Void": "None",
				"System.Array": "List",
				"System.Collections.ArrayList": "List",
				"System.Collections.IList": "List",
				"System.Collections.Generic.List": "List",
				"System.Collections.Generic.IList": "List",
				"System.Enum": "Enum",
				"System.Exception": "Exception",
				"System.AggregateException": "Exception",
				"System.Data.DataRow": "List",
				"System.Data.DataColumn": "List",
				"System.Data.DataColumnChangeEventArgs": "Any",
				"System.Nullable`1[System.Int32]": "Any",
				"System.Action`1[System.Threading.Tasks.Task[]]": "Any",
				"DbDataReader": "Any",
				"ConfiguredTaskAwaiter": "Any",
				"TInput": "Any",
				"TOutput": "Any",
				"TResult": "Any",
				"IntPtr": "Any",
				"StandardValuesCollection": "Any",
				"System.Text.Encoding": "Any",
				"System.Uri": "Any",
				"System.Net.ICredentials": "Any",
				"System.Xml.XPath.XPathNavigator": "Any",
				"YieldAwaiter": "Any",
				"TypedReference": "Any",
				"MarshalByRefObject": "Any",
				"ReadOnlyCollectionBase": "Any",
				"Microsoft.Win32.SafeHandles.SafeWaitHandle": "Any",
				"EventArgs": "Any",
				"System.Threading.Tasks.Task`1[System.Int32]": "System.Threading.Tasks.Task",
				"System.Collections.Generic.IEnumerable": "List",
				"Hashtable": "Any",
				"System.Delegate": "Any",
				"System.IAsyncResult": "Any",
				"System.TypeCode": "Any",
				"System.MulticastDelegate": "Any",
				"MulticastDelegate": "Any",
				"ValueType": "Any",
				"System.EventHandler": "Any",
				"System.IFormatProvider": "Any",
				"System.AsyncCallback": "Any",
				"System.Data.Common.DbDataReader": "Any",
				"T": "Any",
				"System.Collections.Generic.IComparer": "Any",
				"IonPolarity": "Any",
				"SmoothingFunctionType": "Any",
				"SortDirection": "Any",
				**extra_mappings,
				}

	def convert_type(self, csharp_type: str) -> str:
		csharp_type = re.sub(" ByRef$", '', csharp_type)

		if csharp_type in self.type_mapping:
			return self.type_mapping[csharp_type]

		elif csharp_type.endswith("[]"):
			if csharp_type[:-2] in self.type_mapping:
				return f"List[{self.type_mapping[csharp_type[:-2]]}]"
			else:
				print(f"Warning: Unknown type {csharp_type!r}")
				return f"List[{csharp_type[:-2]}]"

		elif generic_type_regex.match(csharp_type):
			m = generic_type_regex.match(csharp_type)

			container_type = self.convert_type(m.group(1))  # type: ignore

			if container_type == "Any":
				return "Any"

			inner_type = ", ".join(self.convert_type(t.strip()) for t in m.group(3).split(","))  # type: ignore

			return f"{container_type}[{inner_type}]"

		elif csharp_type.startswith("System."):
			return csharp_type

		elif csharp_type.startswith("Microsoft.Win32."):
			return "Any"

		else:
			# print(generic_type_regex.match(csharp_type))
			raise KeyError(f"Unknown type: {csharp_type}")



from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Type

import System
import System.Collections
import System.Configuration
import System.Configuration.Assemblies
import System.Data
import System.Globalization
import System.IO
import System.Reflection
import System.Runtime
import System.Runtime.CompilerServices
import System.Runtime.InteropServices
import System.Runtime.Remoting
import System.Runtime.Serialization
import System.Security
import System.Security.AccessControl
import System.Security.Cryptography
import System.Security.Cryptography.X509Certificates
import System.Threading
import System.Threading.Tasks
import System.Xml
import System.Xml.Schema
import System.Xml.Serialization
from System.ComponentModel import MarshalByValueComponent


class Evidence:

    def AddAssembly(self, _: object) -> None: ...

    def AddAssemblyEvidence(self, *args, **kwargs) -> Any: ...

    def AddHost(self, _: object) -> None: ...

    def AddHostEvidence(self, *args, **kwargs) -> Any: ...

    def Clear(self) -> None: ...

    def Clone(self) -> Evidence: ...

    def CopyTo(self, _: List, __: int) -> None: ...

    @property
    def Count(self): ...

    @Count.setter
    def Count(self): ...

    def Equals(self, _: object, __: object) -> bool: ...

    def Finalize(self) -> None: ...

    def GetAssemblyEnumerator(self) -> System.Collections.IEnumerator: ...

    def GetAssemblyEvidence(self, *args, **kwargs) -> Any: ...

    def GetEnumerator(self) -> System.Collections.IEnumerator: ...

    def GetHashCode(self) -> int: ...

    def GetHostEnumerator(self) -> System.Collections.IEnumerator: ...

    def GetHostEvidence(self, *args, **kwargs) -> Any: ...

    def GetType(self) -> Type: ...

    @property
    def IsReadOnly(self): ...

    @IsReadOnly.setter
    def IsReadOnly(self): ...

    @property
    def IsSynchronized(self): ...

    @IsSynchronized.setter
    def IsSynchronized(self): ...

    @property
    def Locked(self): ...

    @Locked.setter
    def Locked(self): ...

    def MemberwiseClone(self) -> object: ...

    def Merge(self, _: Evidence) -> None: ...

    def Overloads(self, *args, **kwargs) -> Any: ...

    def ReferenceEquals(self, _: object, __: object) -> bool: ...

    def RemoveType(self, _: Type) -> None: ...

    @property
    def SyncRoot(self): ...

    @SyncRoot.setter
    def SyncRoot(self): ...

    def ToString(self) -> str: ...

    def __init__(self, *args, **kwargs) -> Any: ...

    def get_Count(self) -> int: ...

    def get_IsReadOnly(self) -> bool: ...

    def get_IsSynchronized(self) -> bool: ...

    def get_Locked(self) -> bool: ...

    def get_SyncRoot(self) -> object: ...

    def set_Locked(self, _: bool) -> None: ...
